@@ -20,8 +20,10 @@
 #include <parser/parse_relation.h>
 #include <parser/parsetree.h>
 #include <planner/planner.h>
+#include <storage/lockdefs.h>
 #include <utils/builtins.h>
 #include <utils/lsyscache.h>
+#include <utils/syscache.h>
 #include <utils/typcache.h>
 
 #include <planner.h>
@@ -34,6 +36,7 @@
 #include "debug_assert.h"
 #include "import/allpaths.h"
 #include "import/planner.h"
+#include "nodes/columnar_index_scan/columnar_index_scan.h"
 #include "nodes/columnar_scan/columnar_scan.h"
 #include "nodes/columnar_scan/planner.h"
 #include "nodes/columnar_scan/qual_pushdown.h"
@@ -145,16 +148,16 @@ append_ec_for_seqnum(PlannerInfo *root, const CompressionInfo *info, const SortI
 }
 
 static EquivalenceClass *
-append_ec_for_metadata_col(PlannerInfo *root, const CompressionInfo *info, Var *var, PathKey *pk)
+append_ec_for_metadata_col(PlannerInfo *root, const CompressionInfo *info, Expr *expr, PathKey *pk)
 {
 	MemoryContext oldcontext = MemoryContextSwitchTo(root->planner_cxt);
 	EquivalenceMember *em = makeNode(EquivalenceMember);
 
-	em->em_expr = (Expr *) var;
+	em->em_expr = expr;
 	em->em_relids = bms_make_singleton(info->compressed_rel->relid);
 	em->em_is_const = false;
 	em->em_is_child = false;
-	em->em_datatype = var->vartype;
+	em->em_datatype = exprType((Node *) expr);
 	EquivalenceClass *ec = makeNode(EquivalenceClass);
 	ec->ec_opfamilies = pk->pk_eclass->ec_opfamilies;
 	ec->ec_collation = pk->pk_eclass->ec_collation;
@@ -292,7 +295,10 @@ build_compressed_scan_pathkeys(const SortInfo *sort_info, PlannerInfo *root, Lis
 				pk = lfirst(lc);
 				expr = ts_find_em_expr_for_rel(pk->pk_eclass, info->chunk_rel);
 
-				Assert(expr != NULL && IsA(expr, Var));
+				Assert(expr);
+				Oid opcintype = exprType((Node *) expr);
+				Oid collation = exprCollation((Node *) expr);
+				expr = (Expr *) strip_implicit_coercions((Node *) expr);
 				var = castNode(Var, expr);
 				Assert(var->varattno > 0);
 
@@ -320,26 +326,30 @@ build_compressed_scan_pathkeys(const SortInfo *sort_info, PlannerInfo *root, Lis
 					nulls_first = orderby_nullsfirst;
 				}
 
-				var = makeVar(info->compressed_rel->relid,
-							  varattno,
-							  var->vartype,
-							  var->vartypmod,
-							  var->varcollid,
-							  var->varlevelsup);
-				EquivalenceClass *min_ec = append_ec_for_metadata_col(root, info, var, pk);
+				Var *metadata_var = makeVar(info->compressed_rel->relid,
+											varattno,
+											var->vartype,
+											var->vartypmod,
+											var->varcollid,
+											var->varlevelsup);
+				Expr *min_expr =
+					canonicalize_ec_expression((Expr *) metadata_var, opcintype, collation);
+				EquivalenceClass *min_ec = append_ec_for_metadata_col(root, info, min_expr, pk);
 				PathKey *min =
 					make_canonical_pathkey(root, min_ec, pk->pk_opfamily, strategy, nulls_first);
 				required_compressed_pathkeys = lappend(required_compressed_pathkeys, min);
 
 				varattno =
 					get_attnum(info->compressed_rte->relid, column_segment_max_name(orderby_index));
-				var = makeVar(info->compressed_rel->relid,
-							  varattno,
-							  var->vartype,
-							  var->vartypmod,
-							  var->varcollid,
-							  var->varlevelsup);
-				EquivalenceClass *max_ec = append_ec_for_metadata_col(root, info, var, pk);
+				metadata_var = makeVar(info->compressed_rel->relid,
+									   varattno,
+									   var->vartype,
+									   var->vartypmod,
+									   var->varcollid,
+									   var->varlevelsup);
+				Expr *max_expr =
+					canonicalize_ec_expression((Expr *) metadata_var, opcintype, collation);
+				EquivalenceClass *max_ec = append_ec_for_metadata_col(root, info, max_expr, pk);
 				PathKey *max =
 					make_canonical_pathkey(root, max_ec, pk->pk_opfamily, strategy, nulls_first);
 
@@ -449,8 +459,8 @@ set_compressed_baserel_size_estimates(PlannerInfo *root, RelOptInfo *rel,
 	 * First, build the correspondence of min metadata attno -> max metadata
 	 * attno for all minmax metadata.
 	 */
-	AttrNumber *storage =
-		palloc0(2 * sizeof(AttrNumber) * compression_info->compressed_rel->max_attr);
+	const int storage_elements = 2 * (compression_info->compressed_rel->max_attr + 1);
+	AttrNumber *storage = palloc0(storage_elements * sizeof(*storage));
 	SelectivityEstimationContext context = {
 		.min_to_max = &storage[0],
 		.max_to_min = &storage[compression_info->compressed_rel->max_attr],
@@ -502,6 +512,9 @@ set_compressed_baserel_size_estimates(PlannerInfo *root, RelOptInfo *rel,
 			continue;
 		}
 
+		Assert(&context.min_to_max[min_attno] < &storage[storage_elements]);
+		Assert(&context.max_to_min[max_attno] < &storage[storage_elements]);
+
 		context.min_to_max[min_attno] = max_attno;
 		context.max_to_min[max_attno] = min_attno;
 	}
@@ -623,6 +636,8 @@ build_compressioninfo(PlannerInfo *root, const Hypertable *ht, const Chunk *chun
 		info->parent_relids = find_childrel_parents(root, chunk_rel);
 	}
 
+	info->chunk_status = chunk->fd.status;
+
 	return info;
 }
 
@@ -631,22 +646,20 @@ build_compressioninfo(PlannerInfo *root, const Hypertable *ht, const Chunk *chun
  * Postgres statistics for _ts_meta_count column.
  * Returns TARGET_COMPRESSED_BATCH_SIZE when no pg_statistic entry exists.
  */
-static double
-estimate_compressed_batch_size(PlannerInfo *root, const CompressionInfo *compression_info)
+double
+ts_columnar_estimate_compressed_batch_size(const Oid relid)
 {
-	AttrNumber attnum = get_attnum(compression_info->compressed_rte->relid, "_ts_meta_count");
+	AttrNumber attnum = get_attnum(relid, "_ts_meta_count");
 	if (attnum == InvalidAttrNumber)
 		return TARGET_COMPRESSED_BATCH_SIZE;
 
-	Var *var = makeVar(compression_info->compressed_rel->relid, attnum, INT4OID, -1, InvalidOid, 0);
-
 	/* fetch statistics */
-	VariableStatData vardata;
-	examine_variable(root, (Node *) var, 0, &vardata);
-
-	if (!HeapTupleIsValid(vardata.statsTuple))
+	HeapTuple statsTuple = SearchSysCache3(STATRELATTINH,
+										   ObjectIdGetDatum(relid),
+										   Int16GetDatum(attnum),
+										   BoolGetDatum(false));
+	if (!HeapTupleIsValid(statsTuple))
 	{
-		ReleaseVariableStats(vardata);
 		return TARGET_COMPRESSED_BATCH_SIZE;
 	}
 
@@ -656,7 +669,7 @@ estimate_compressed_batch_size(PlannerInfo *root, const CompressionInfo *compres
 	/* exact MCV contribution */
 	AttStatsSlot mcvslot;
 	if (get_attstatsslot(&mcvslot,
-						 vardata.statsTuple,
+						 statsTuple,
 						 STATISTIC_KIND_MCV,
 						 InvalidOid,
 						 ATTSTATSSLOT_VALUES | ATTSTATSSLOT_NUMBERS))
@@ -676,7 +689,7 @@ estimate_compressed_batch_size(PlannerInfo *root, const CompressionInfo *compres
 	/* histogram contribution */
 	AttStatsSlot histslot;
 	if (get_attstatsslot(&histslot,
-						 vardata.statsTuple,
+						 statsTuple,
 						 STATISTIC_KIND_HISTOGRAM,
 						 InvalidOid,
 						 ATTSTATSSLOT_VALUES))
@@ -696,7 +709,7 @@ estimate_compressed_batch_size(PlannerInfo *root, const CompressionInfo *compres
 		free_attstatsslot(&histslot);
 	}
 
-	ReleaseVariableStats(vardata);
+	ReleaseSysCache(statsTuple);
 
 	const double final_result = mcv_sum + hist_sum;
 	if (final_result == 0)
@@ -1135,7 +1148,7 @@ ts_columnar_scan_generate_paths(PlannerInfo *root, RelOptInfo *chunk_rel, const
 	 * statistics.
 	 */
 	compression_info->compressed_batch_size =
-		estimate_compressed_batch_size(root, compression_info);
+		ts_columnar_estimate_compressed_batch_size(compression_info->compressed_rte->relid);
 
 	/*
 	 * Estimate the size of decompressed chunk based on the compressed chunk.
@@ -1224,6 +1237,8 @@ ts_columnar_scan_generate_paths(PlannerInfo *root, RelOptInfo *chunk_rel, const
 	foreach (compressed_cell, compressed_rel->partial_pathlist)
 	{
 		Path *compressed_path = lfirst(compressed_cell);
+		/* Partial parameterized paths are not supported */
+		Assert(bms_is_empty(PATH_REQ_OUTER(compressed_path)));
 		List *decompressed_paths = build_on_single_compressed_path(root,
 																   chunk,
 																   chunk_rel,
@@ -1356,6 +1371,22 @@ build_on_single_compressed_path(PlannerInfo *root, const Chunk *chunk, RelOptInf
 						"time")));
 	}
 
+	/*
+	 * Try to create a ColumnarIndexScan path for the metadata-only optimization.
+	 */
+	{
+		ColumnarIndexScanPath *index_scan_path =
+			ts_columnar_index_scan_path_create(root,
+											   chunk,
+											   chunk_rel,
+											   compression_info,
+											   compressed_path);
+		if (index_scan_path)
+		{
+			decompressed_paths = lappend(decompressed_paths, index_scan_path);
+		}
+	}
+
 	/*
 	 * If we can push down the sort below the ColumnarScan node, we set the
 	 * pathkeys of the decompress node to the decompressed_sort_pathkeys. We
@@ -2043,6 +2074,7 @@ add_segmentby_to_equivalence_class(PlannerInfo *root, EquivalenceClass *cur_ec,
 	{
 		cur_em = (EquivalenceMember *) lfirst(lc);
 #endif
+		Node *node;
 		Expr *child_expr;
 		Relids new_relids;
 		Var *var;
@@ -2050,11 +2082,10 @@ add_segmentby_to_equivalence_class(PlannerInfo *root, EquivalenceClass *cur_ec,
 
 		/* only consider EquivalenceMembers that are Vars, possibly with RelabelType, of the
 		 * uncompressed chunk */
-		var = (Var *) cur_em->em_expr;
-		while (var && IsA(var, RelabelType))
-			var = (Var *) ((RelabelType *) var)->arg;
-		if (!(var && IsA(var, Var)))
+		node = strip_implicit_coercions((Node *) cur_em->em_expr);
+		if (!(node && IsA(node, Var)))
 			continue;
+		var = castNode(Var, node);
 
 		/*
 		 * We want to base our equivalence member on the hypertable equivalence
@@ -2228,8 +2259,14 @@ columnar_scan_add_plannerinfo(PlannerInfo *root, CompressionInfo *info, const Ch
 	/*
 	 * Add the compressed chunk to the baserel cache. Note that it belongs to
 	 * a different hypertable, the internal compression table.
+	 *
+	 * Ensure we do not grab a slice lock because that will assign a transaction ID that could
+	 * unnecessarily block other operations.
 	 */
-	const Chunk *compressed_chunk = ts_chunk_get_by_relid(info->settings->fd.compress_relid, true);
+	const Chunk *compressed_chunk = ts_chunk_get_by_relid_locked(info->settings->fd.compress_relid,
+																 AccessShareLock,
+																 NULL,
+																 true);
 	ts_add_baserel_cache_entry_for_chunk(info->settings->fd.compress_relid,
 										 ts_planner_get_hypertable(compressed_chunk
 																	   ->hypertable_relid,
@@ -2268,6 +2305,7 @@ columnar_scan_add_plannerinfo(PlannerInfo *root, CompressionInfo *info, const Ch
 	 */
 	Assert(info->single_chunk || chunk_rel->top_parent_relids != NULL);
 	compressed_rel->top_parent_relids = bms_copy(chunk_rel->top_parent_relids);
+	compressed_rel->lateral_relids = bms_copy(chunk_rel->lateral_relids);
 
 	root->simple_rel_array[compressed_index] = compressed_rel;
 	info->compressed_rel = compressed_rel;
@@ -2311,7 +2349,6 @@ columnar_scan_add_plannerinfo(PlannerInfo *root, CompressionInfo *info, const Ch
 		 */
 		root->append_rel_array[compressed_rel->relid] = makeNode(AppendRelInfo);
 		root->append_rel_array[compressed_rel->relid]->parent_relid = info->ht_rel->relid;
-		compressed_rel->top_parent_relids = chunk_rel->top_parent_relids;
 	}
 }
 
@@ -2365,6 +2402,7 @@ columnar_scan_path_create(PlannerInfo *root, const CompressionInfo *compression_
 
 	path->custom_path.custom_paths = list_make1(compressed_path);
 	path->reverse = false;
+	path->chunk_status = compression_info->chunk_status;
 	path->required_compressed_pathkeys = NIL;
 	cost_columnar_scan(root, compression_info, &path->custom_path.path, compressed_path);
 
@@ -2380,6 +2418,10 @@ create_compressed_scan_paths(PlannerInfo *root, RelOptInfo *compressed_rel,
 							 const CompressionInfo *compression_info, const SortInfo *sort_info)
 {
 	Path *compressed_path;
+	Relids required_outer = compressed_rel->lateral_relids;
+
+	/* Must have same lateral relids as the chunk hypertable */
+	Assert(bms_equal(required_outer, compression_info->chunk_rel->lateral_relids));
 
 	/* clamp total_table_pages to 10 pages since this is the
 	 * minimum estimate for number of pages.
@@ -2388,7 +2430,7 @@ create_compressed_scan_paths(PlannerInfo *root, RelOptInfo *compressed_rel,
 	root->total_table_pages += Max(compressed_rel->pages, 10);
 
 	/* create non parallel scan path */
-	compressed_path = create_seqscan_path(root, compressed_rel, NULL, 0);
+	compressed_path = create_seqscan_path(root, compressed_rel, required_outer, 0);
 	add_path(compressed_rel, compressed_path);
 
 	/*
@@ -2398,8 +2440,11 @@ create_compressed_scan_paths(PlannerInfo *root, RelOptInfo *compressed_rel,
 	 * tables, so that they don't prevent parallelism in the entire append plan.
 	 * See compute_parallel_workers(). This also applies to the creation of
 	 * index paths below.
+	 *
+	 * Parameterized rels that depend on an outer rel are not allowed to form partial
+	 * sequential scan paths
 	 */
-	if (compressed_rel->consider_parallel)
+	if (compressed_rel->consider_parallel && required_outer == NULL)
 	{
 		int parallel_workers = compute_parallel_worker(compressed_rel,
 													   compressed_rel->pages,
@@ -2540,29 +2585,25 @@ find_const_segmentby(RelOptInfo *chunk_rel, const CompressionInfo *info)
 			if (IsA(ri->clause, OpExpr) && list_length(castNode(OpExpr, ri->clause)->args) == 2)
 			{
 				OpExpr *op = castNode(OpExpr, ri->clause);
-				Var *lvar, *rvar, *var;
+				Node *lnode, *rnode;
+				Var *var;
 				Expr *other;
 
 				if (op->opretset)
 					continue;
 
-				lvar = linitial(op->args);
-				while (lvar && IsA(lvar, RelabelType))
-					lvar = (Var *) ((RelabelType *) lvar)->arg;
+				lnode = strip_implicit_coercions(linitial(op->args));
+				rnode = strip_implicit_coercions(lsecond(op->args));
 
-				rvar = lsecond(op->args);
-				while (rvar && IsA(rvar, RelabelType))
-					rvar = (Var *) ((RelabelType *) rvar)->arg;
-
-				Assert(lvar && rvar);
-				if (IsA(lvar, Var))
+				Assert(lnode && rnode);
+				if (IsA(lnode, Var))
 				{
-					var = castNode(Var, lvar);
+					var = castNode(Var, lnode);
 					other = lsecond(op->args);
 				}
-				else if (IsA(rvar, Var))
+				else if (IsA(rnode, Var))
 				{
-					var = castNode(Var, rvar);
+					var = castNode(Var, rnode);
 					other = linitial(op->args);
 				}
 				else
@@ -2576,7 +2617,41 @@ find_const_segmentby(RelOptInfo *chunk_rel, const CompressionInfo *info)
 					TypeCacheEntry *tce = lookup_type_cache(var->vartype, TYPECACHE_EQ_OPR);
 
 					if (op->opno != tce->eq_opr)
-						continue;
+					{
+						/* Issue #9066: check if our OpExpr is still an equality (Var = Const)
+						 * for Var and Const of different types
+						 */
+#if PG18_GE
+						List *opinfos = get_op_index_interpretation(op->opno);
+#else
+						List *opinfos = get_op_btree_interpretation(op->opno);
+#endif
+						ListCell *lc;
+						bool equality = false;
+						foreach (lc, opinfos)
+						{
+#if PG18_GE
+							OpIndexInterpretation *opinfo = (OpIndexInterpretation *) lfirst(lc);
+							if (opinfo->cmptype == COMPARE_EQ)
+#else
+							OpBtreeInterpretation *opinfo = (OpBtreeInterpretation *) lfirst(lc);
+							if (opinfo->strategy == BTEqualStrategyNumber)
+#endif
+							{
+								Oid mixed_type_eqop = get_opfamily_member(opinfo->opfamily_id,
+																		  var->vartype,
+																		  exprType((Node *) other),
+																		  BTEqualStrategyNumber);
+								if (op->opno == mixed_type_eqop)
+								{
+									equality = true;
+									break;
+								}
+							}
+						}
+						if (!equality)
+							continue;
+					}
 
 					if (bms_is_member(var->varattno, info->chunk_segmentby_attnos))
 						segmentby_columns = bms_add_member(segmentby_columns, var->varattno);
@@ -2602,16 +2677,14 @@ match_pathkeys_to_compression_orderby(List *pathkeys, List *chunk_em_exprs,
 	{
 		compressed_pk_index++;
 		PathKey *pk = list_nth_node(PathKey, pathkeys, i);
-		Expr *expr = (Expr *) list_nth(chunk_em_exprs, i);
-		while (expr && IsA(expr, RelabelType))
-			expr = ((RelabelType *) expr)->arg;
+		Node *node = strip_implicit_coercions((Node *) list_nth(chunk_em_exprs, i));
 
-		if (expr == NULL || !IsA(expr, Var))
+		if (node == NULL || !IsA(node, Var))
 		{
 			return false;
 		}
 
-		Var *var = castNode(Var, expr);
+		Var *var = castNode(Var, node);
 
 		if (var->varattno <= 0)
 		{
@@ -2686,7 +2759,7 @@ match_pathkeys_to_compression_orderby(List *pathkeys, List *chunk_em_exprs,
 }
 
 /*
- * Check if we can push down the sort below the ColumnarSacn node and fill
+ * Check if we can push down the sort below the ColumnarScan node and fill
  * SortInfo accordingly
  *
  * The following conditions need to be true for pushdown:
@@ -2753,12 +2826,13 @@ build_sortinfo(PlannerInfo *root, const Chunk *chunk, RelOptInfo *chunk_rel,
 	cost_qual_eval(&sort_info.decompressed_sort_pathkeys_cost, sort_pathkey_exprs, root);
 
 	/*
-	 * Next, check if we can push the sort down to the uncompressed part.
+	 * Next, check if we can push the sort down to the compressed part.
 	 *
-	 * Not possible if the chunk is unordered.
+	 * Batch sorted merge optimization is enabled for unordered chunks
+	 * because we do merge the batches at execution time which
+	 * only relies that the batches themselves are sorted which is
+	 * always the case.
 	 */
-	if (ts_chunk_is_unordered(chunk))
-		return sort_info;
 
 	/* all segmentby columns need to be prefix of pathkeys */
 	int i = 0;
@@ -2781,13 +2855,11 @@ build_sortinfo(PlannerInfo *root, const Chunk *chunk, RelOptInfo *chunk_rel,
 		{
 			Assert(bms_num_members(segmentby_columns) <= compression_info->num_segmentby_columns);
 
-			Expr *expr = (Expr *) list_nth(chunk_em_exprs, i);
-			while (expr && IsA(expr, RelabelType))
-				expr = ((RelabelType *) expr)->arg;
+			Node *node = strip_implicit_coercions((Node *) list_nth(chunk_em_exprs, i));
 
-			if (expr == NULL || !IsA(expr, Var))
+			if (node == NULL || !IsA(node, Var))
 				break;
-			var = castNode(Var, expr);
+			var = castNode(Var, node);
 
 			if (var->varattno <= 0)
 				break;
@@ -2799,6 +2871,16 @@ build_sortinfo(PlannerInfo *root, const Chunk *chunk, RelOptInfo *chunk_rel,
 			segmentby_columns = bms_add_member(segmentby_columns, var->varattno);
 		}
 
+		/*
+		 * Pathkeys satisfied by sorting the compressed data on segmentby columns.
+		 * Can use compressed sort on segmentby cols for unordered chunks as well.
+		 */
+		if (i == list_length(pathkeys))
+		{
+			sort_info.use_compressed_sort = true;
+			return sort_info;
+		}
+
 		/*
 		 * If pathkeys still has items, but we didn't find all segmentby columns,
 		 * we cannot satisfy these pathkeys by sorting the compressed chunk table.
@@ -2823,14 +2905,12 @@ build_sortinfo(PlannerInfo *root, const Chunk *chunk, RelOptInfo *chunk_rel,
 		}
 	}
 
-	if (i == list_length(pathkeys))
-	{
-		/*
-		 * Pathkeys satisfied by sorting the compressed data on segmentby columns.
-		 */
-		sort_info.use_compressed_sort = true;
+	/*
+	 * Cannot push down sort on non-segmentby columns
+	 * if the chunk has batches overlapping on orderby columns
+	 */
+	if (ts_chunk_is_unordered(chunk))
 		return sort_info;
-	}
 
 	/*
 	 * Pathkeys includes columns past segmentby columns, so we need sequence_num
