@@ -15,6 +15,7 @@
 
 #include "columnar_scan.h"
 #include "compression/create.h"
+#include "compression/sparse_index_bloom1.h"
 #include "custom_type_cache.h"
 #include "guc.h"
 #include "ts_catalog/array_utils.h"
@@ -372,7 +373,21 @@ expr_fetch_bloom1_metadata(QualPushdownContext *context, Expr *expr, AttrNumber
 													 context->chunk_rte->relid,
 													 var->varattno,
 													 context->compressed_rte->relid,
-													 "bloom1");
+													 bloom1_column_prefix);
+
+	if (*bloom1_attno == InvalidAttrNumber && ts_guc_read_legacy_bloom1_v1)
+	{
+		/*
+		 * The version 1 of bloom1 indexes is disabled by default because its
+		 * hashing was dependent on build options leading to corrupt indexes,
+		 * but can be enabled manually.
+		 */
+		*bloom1_attno = compressed_column_metadata_attno(context->settings,
+														 context->chunk_rte->relid,
+														 var->varattno,
+														 context->compressed_rte->relid,
+														 "bloom1");
+	}
 }
 
 static void *
@@ -467,6 +482,22 @@ pushdown_op_to_segment_meta_bloom1(QualPushdownContext *context, OpExpr *orig_op
 	}
 	Assert(pushed_down_rightop != NULL);
 
+	/*
+	 * We can have cross-type equality operator, but in this case the our hashes
+	 * or Postgres hashes for the respective types are guaranteed to have the
+	 * same result for both types, so we don't need any type conversion here.
+	 * The only special case is composite types. The right-hand constant would
+	 * have the anonymous type "record" and would be compared polymorphically
+	 * at runtime with the record_eq() function. However, this type doesn't have
+	 * an extended hash function. Just refuse to work with it.
+	 */
+	const Oid compared_type = exprType((Node *) pushed_down_rightop);
+	if (compared_type == RECORDOID)
+	{
+		context->can_pushdown = false;
+		return orig_opexpr;
+	}
+
 	/*
 	 * var = expr implies bloom1_contains(var_bloom, expr).
 	 */
@@ -491,6 +522,122 @@ pushdown_op_to_segment_meta_bloom1(QualPushdownContext *context, OpExpr *orig_op
 								 COERCE_EXPLICIT_CALL);
 }
 
+/*
+ * Try to transform x = any(array[]) into bloom1_contains_any(bloom_x, array[]).
+ */
+static void *
+pushdown_saop_bloom1(QualPushdownContext *context, ScalarArrayOpExpr *orig_saop)
+{
+	/*
+	 * This always requires rechecking the decompressed data.
+	 */
+	context->needs_recheck = true;
+
+	if (!orig_saop->useOr)
+	{
+		context->can_pushdown = false;
+		return orig_saop;
+	}
+
+	List *expr_args = orig_saop->args;
+	Assert(list_length(expr_args) == 2);
+	Expr *orig_leftop = linitial(expr_args);
+	Expr *orig_rightop = lsecond(expr_args);
+
+	if (IsA(orig_leftop, RelabelType))
+		orig_leftop = ((RelabelType *) orig_leftop)->arg;
+	if (IsA(orig_rightop, RelabelType))
+		orig_rightop = ((RelabelType *) orig_rightop)->arg;
+
+	/*
+	 * For scalar array operation, we expect a var on the left side.
+	 */
+	AttrNumber bloom1_attno = InvalidAttrNumber;
+	expr_fetch_bloom1_metadata(context, orig_leftop, &bloom1_attno);
+	if (bloom1_attno == InvalidAttrNumber)
+	{
+		/* No metadata for left operand. */
+		context->can_pushdown = false;
+		return orig_saop;
+	}
+
+	Var *var_with_segment_meta = castNode(Var, orig_leftop);
+
+	/*
+	 * Play it safe and don't push down if the operator collation doesn't match
+	 * the column collation.
+	 */
+	Oid op_collation = orig_saop->inputcollid;
+	if (var_with_segment_meta->varcollid != op_collation)
+	{
+		context->can_pushdown = false;
+		return orig_saop;
+	}
+
+	/*
+	 * We cannot use bloom filters for non-deterministic collations.
+	 */
+	if (OidIsValid(op_collation) && !get_collation_isdeterministic(op_collation))
+	{
+		context->can_pushdown = false;
+		return orig_saop;
+	}
+
+	/*
+	 * We only support hashable equality operators.
+	 */
+	const Oid op_oid = orig_saop->opno;
+	TypeCacheEntry *tce =
+		lookup_type_cache(var_with_segment_meta->vartype, TYPECACHE_HASH_OPFAMILY);
+	const int strategy = get_op_opfamily_strategy(op_oid, tce->hash_opf);
+	if (strategy != HTEqualStrategyNumber)
+	{
+		context->can_pushdown = false;
+		return orig_saop;
+	}
+
+	/*
+	 * The hash equality operators are supposed to be strict.
+	 */
+	Assert(op_strict(op_oid));
+
+	/*
+	 * Check if the righthand expression is safe to push down. We cannot combine
+	 * it with the original operator if there can be false negatives.
+	 */
+	QualPushdownContext tmp_context = copy_context(context);
+	Expr *pushed_down_rightop = (Expr *) qual_pushdown_mutator((Node *) orig_rightop, &tmp_context);
+	if (!tmp_context.can_pushdown || tmp_context.needs_recheck)
+	{
+		context->can_pushdown = false;
+		return orig_saop;
+	}
+	Assert(pushed_down_rightop != NULL);
+
+	/*
+	 * var = any(array) implies bloom1_contains_any(var_bloom, array).
+	 */
+	Var *bloom_var = makeVar(context->compressed_rel->relid,
+							 bloom1_attno,
+							 ts_custom_type_cache_get(CUSTOM_TYPE_BLOOM1)->type_oid,
+							 -1,
+							 InvalidOid,
+							 0);
+
+	Oid func = LookupFuncName(list_make2(makeString("_timescaledb_functions"),
+										 makeString("bloom1_contains_any")),
+							  /* nargs = */ -1,
+							  /* argtypes = */ (void *) -1,
+							  /* missing_ok = */ false);
+
+	return makeFuncExpr(func,
+						BOOLOID,
+						list_make2(bloom_var, pushed_down_rightop),
+						/* funccollid = */ InvalidOid,
+						/* inputcollid = */ InvalidOid,
+						COERCE_EXPLICIT_CALL);
+}
+
 static bool
 contain_volatile_functions_checker(Oid func_id, void *context)
 {
@@ -655,6 +802,23 @@ qual_pushdown_mutator(Node *orig_node, QualPushdownContext *context)
 				return pushed_down;
 			}
 
+			ScalarArrayOpExpr *saop = castNode(ScalarArrayOpExpr, orig_node);
+
+			/*
+			 * Try to transform x = any(array[]) into
+			 * bloom1_contains_any(bloom_x, array[]).
+			 */
+			if (ts_guc_enable_sparse_index_bloom)
+			{
+				tmp_context = *context;
+				pushed_down = pushdown_saop_bloom1(&tmp_context, saop);
+				if (tmp_context.can_pushdown)
+				{
+					context->needs_recheck |= tmp_context.needs_recheck;
+					return pushed_down;
+				}
+			}
+
 			/*
 			 * No other ways to push it down, so consider it failed.
 			 */
