@@ -28,28 +28,28 @@
 #include "compat/compat.h"
 #include "guc.h"
 #include "nodes/chunk_append/chunk_append.h"
+#include "nodes/columnar_scan/columnar_scan.h"
 #include "nodes/constraint_aware_append/constraint_aware_append.h"
-#include "nodes/decompress_chunk/decompress_chunk.h"
 #include "nodes/skip_scan/skip_scan.h"
 #include <import/planner.h>
 
 #include <math.h>
 
-typedef struct SkipScanPath
+typedef struct SkipKeyInfo
 {
-	CustomPath cpath;
-	IndexPath *index_path;
-
 	/* Index clause which we'll use to skip past elements we've already seen */
 	RestrictInfo *skip_clause;
 
+	/* Is this key guaranteed to be not null? */
+	bool notnull;
+
 	/* attribute number of the distinct column on the table/chunk which provides comparison value
 	 * for Skip qual */
 	AttrNumber distinct_attno;
 
 	/* attribute number of the Skip qual comparison column on the indexed table/chunk
 	 * "indexed_column_attno = distinct_attno" for (SkipScan <- Index Scan) scenario,
-	 * it can be different for (SkipScan <- DecompressChunk <- compressed Index Scan) scenario,
+	 * it can be different for (SkipScan <- ColumnarScan <- compressed Index Scan) scenario,
 	 * in that case "indexed_column_attno" is the attribute number of the compressed chunk column
 	 * corresponding to the distinct column "distinct_attno" on the decompressed chunk consumed by
 	 * SkipScan
@@ -60,26 +60,44 @@ typedef struct SkipScanPath
 	AttrNumber scankey_attno;
 	int distinct_typ_len;
 	bool distinct_by_val;
-	/* Var referencing the distinct column on the relation */
-	Var *distinct_var;
+
+	/* InvalidOid for the last skip key, always invalid for one-key SkipScan
+	 * For N-key SkipScan default quals are (sk1 = p1), (sk2 = p2), .. (sk_n > p_n),
+	 * we'll switch to (sk_i > p_i) when no more values for (sk_i+1 > p_i+1),
+	 * so we will store "=" along with ">" comparator for keys 1..N-1.
+	 */
+	Oid eqcomp;
+} SkipKeyInfo;
+
+typedef struct SkipScanPath
+{
+	CustomPath cpath;
+	IndexPath *index_path;
+
+	/* List of skip column attributes for each skip key */
+	List *skipkeyinfo;
+
+	/* Vars referencing the distinct columns on the relation */
+	List *dvars;
 } SkipScanPath;
 
 typedef struct DistinctPathInfo
 {
 	UpperRelationKind stage; /* What kind of Upper distinct path we are dealing with */
 	Path *unique_path;		 /* If not NULL, valid Upper distinct path */
-	Expr *distinct_expr;	 /* If not NULL, valid distinct expression for Upper distinct path */
+	List *
+		distinct_expr; /* If not NULL, list of valid distinct expressions for Upper distinct path */
 } DistinctPathInfo;
 
 static int get_idx_key(IndexOptInfo *idxinfo, AttrNumber attno);
 static List *sort_indexquals(IndexOptInfo *indexinfo, List *quals);
 static OpExpr *fix_indexqual(IndexOptInfo *index, RestrictInfo *rinfo, AttrNumber scankey_attno);
-static bool build_skip_qual(PlannerInfo *root, SkipScanPath *skip_scan_path, IndexPath *index_path,
-							Var *var);
+static bool build_skip_qual(PlannerInfo *root, SkipKeyInfo *skinfo, IndexPath *index_path, Var *var,
+							bool build_eqop);
 static List *build_subpath(PlannerInfo *root, List *subpaths, DistinctPathInfo *dpinfo,
 						   List *top_pathkeys);
-static Var *get_distinct_var(PlannerInfo *root, DistinctPathInfo *dpinfo, Path *child_path,
-							 SkipScanPath *skip_scan_path);
+static Var *get_distinct_var(PlannerInfo *root, Expr *tlexpr, IndexPath *index_path,
+							 Path *child_path, SkipKeyInfo *skinfo);
 static TargetEntry *tlist_member_match_var(Var *var, List *targetlist);
 
 /**************************
@@ -109,7 +127,7 @@ setup_index_plan(CustomScan *skip_plan, Plan *child_plan)
 	{
 		skip_plan->scan = castNode(IndexOnlyScan, child_plan)->scan;
 	}
-	else if (ts_is_decompress_chunk_plan(child_plan))
+	else if (ts_is_columnar_scan_plan(child_plan))
 	{
 		CustomScan *csplan = castNode(CustomScan, plan);
 		skip_plan->scan = csplan->scan;
@@ -131,46 +149,131 @@ skip_scan_plan_create(PlannerInfo *root, RelOptInfo *relopt, CustomPath *best_pa
 	CustomScan *skip_plan = makeNode(CustomScan);
 	IndexPath *index_path = path->index_path;
 
-	OpExpr *op = fix_indexqual(index_path->indexinfo, path->skip_clause, path->scankey_attno);
-
 	Plan *child_plan = linitial(custom_plans);
 	Plan *plan = setup_index_plan(skip_plan, child_plan);
 
-	if (IsA(plan, IndexScan))
-	{
-		IndexScan *idx_plan = castNode(IndexScan, plan);
-		/* we prepend skip qual here so sort_indexquals will put it as first qual for that column */
-		idx_plan->indexqual =
-			sort_indexquals(index_path->indexinfo, lcons(op, idx_plan->indexqual));
-	}
-	else if (IsA(plan, IndexOnlyScan))
-	{
-		IndexOnlyScan *idx_plan = castNode(IndexOnlyScan, plan);
-		/* we prepend skip qual here so sort_indexquals will put it as first qual for that column */
-		idx_plan->indexqual =
-			sort_indexquals(index_path->indexinfo, lcons(op, idx_plan->indexqual));
-	}
-	else
-		elog(ERROR, "unsupported subplan type for SkipScan: %s", ts_get_node_name((Node *) plan));
-
 	skip_plan->scan.plan.targetlist = tlist;
 	skip_plan->custom_scan_tlist = list_copy(tlist);
 	skip_plan->scan.plan.qual = NIL;
 	skip_plan->scan.plan.type = T_CustomScan;
 	skip_plan->methods = &skip_scan_plan_methods;
 	skip_plan->custom_plans = custom_plans;
-	/* get position of distinct column in tuples produced by child scan */
-	TargetEntry *tle = tlist_member_match_var(path->distinct_var, child_plan->targetlist);
 
-	bool nulls_first = index_path->indexinfo->nulls_first[path->scankey_attno - 1];
-	if (index_path->indexscandir == BackwardScanDirection)
-		nulls_first = !nulls_first;
+	/* Setup for SkipScan debug info */
+	StringInfoData debuginfo;
+	RangeTblEntry *indexed_rte = NULL;
+	char *sep = "";
+	if (ts_guc_debug_skip_scan_info)
+	{
+		initStringInfo(&debuginfo);
+		RelOptInfo *indexed_rel = index_path->path.parent;
+		indexed_rte = planner_rt_fetch(indexed_rel->relid, root);
+		Oid indrelid = InvalidOid;
+		if (IsA(plan, IndexScan))
+		{
+			IndexScan *idx_plan = castNode(IndexScan, plan);
+			indrelid = idx_plan->indexid;
+		}
+		else if (IsA(plan, IndexOnlyScan))
+		{
+			IndexOnlyScan *idx_plan = castNode(IndexOnlyScan, plan);
+			indrelid = idx_plan->indexid;
+		}
+		appendStringInfo(&debuginfo, "SkipScan used on %s(", get_rel_name(indrelid));
+	}
+
+	ListCell *lc, *lv;
+	/* List of N-1 equality op Oids for N-key skipscan, stays NIL for one-key skipscan */
+	List *eqcomps = NIL;
+	/* List of N skipkeyinfo Int lists for N-key skipscan */
+	List *skinfos = NIL;
+	forboth (lc, path->skipkeyinfo, lv, path->dvars)
+	{
+		SkipKeyInfo *skinfo = (SkipKeyInfo *) lfirst(lc);
+		Var *dvar = castNode(Var, lfirst(lv));
+		OpExpr *op =
+			fix_indexqual(index_path->indexinfo, skinfo->skip_clause, skinfo->scankey_attno);
+		if (OidIsValid(skinfo->eqcomp))
+			eqcomps = lappend_oid(eqcomps, skinfo->eqcomp);
+
+		if (IsA(plan, IndexScan))
+		{
+			IndexScan *idx_plan = castNode(IndexScan, plan);
+			/* we prepend skip qual here so sort_indexquals will put it as first qual for that
+			 * column */
+			idx_plan->indexqual =
+				sort_indexquals(index_path->indexinfo, lcons(op, idx_plan->indexqual));
+		}
+		else if (IsA(plan, IndexOnlyScan))
+		{
+			IndexOnlyScan *idx_plan = castNode(IndexOnlyScan, plan);
+			/* we prepend skip qual here so sort_indexquals will put it as first qual for that
+			 * column */
+			idx_plan->indexqual =
+				sort_indexquals(index_path->indexinfo, lcons(op, idx_plan->indexqual));
+		}
+		else
+			elog(ERROR,
+				 "unsupported subplan type for SkipScan: %s",
+				 ts_get_node_name((Node *) plan));
+
+		/* get position of distinct column in tuples produced by child scan */
+		TargetEntry *tle = tlist_member_match_var(dvar, child_plan->targetlist);
+
+		SkipKeyNullStatus sknulls;
+		if (skinfo->notnull)
+			sknulls = SK_NOT_NULL;
+		else
+		{
+			bool nulls_first = index_path->indexinfo->nulls_first[skinfo->scankey_attno - 1];
+			if (index_path->indexscandir == BackwardScanDirection)
+				nulls_first = !nulls_first;
+			sknulls = (nulls_first ? SK_NULLS_FIRST : SK_NULLS_LAST);
+		}
+		skinfos = lappend(skinfos,
+						  list_make5_int(tle->resno,
+										 skinfo->distinct_by_val,
+										 skinfo->distinct_typ_len,
+										 sknulls,
+										 skinfo->scankey_attno));
+		/* Debug info about skip key */
+		if (ts_guc_debug_skip_scan_info)
+		{
+			char *attname = get_attname(indexed_rte->relid, skinfo->indexed_column_attno, false);
+			char *sknullstext;
+			switch (sknulls)
+			{
+				case SK_NOT_NULL:
+					sknullstext = "NOT NULL";
+					break;
+				case SK_NULLS_FIRST:
+					sknullstext = "NULLS FIRST";
+					break;
+				case SK_NULLS_LAST:
+					sknullstext = "NULLS LAST";
+					break;
+				default:
+					Assert(false);
+			}
+			appendStringInfo(&debuginfo, "%s%s %s", sep, attname, sknullstext);
+			sep = ", ";
+		}
+	}
+
+	if (ts_guc_debug_skip_scan_info)
+	{
+		appendStringInfoString(&debuginfo, ")");
+		elog(INFO, "%s", debuginfo.data);
+	}
+
+	skip_plan->custom_private = lappend(skip_plan->custom_private, skinfos);
+	/* Don't need equality ops for one-key skipscan */
+	if (eqcomps != NIL)
+	{
+		Assert(list_length(skinfos) > 1);
+		skip_plan->custom_private = lappend(skip_plan->custom_private, eqcomps);
+	}
 
-	skip_plan->custom_private = list_make5_int(tle->resno,
-											   path->distinct_by_val,
-											   path->distinct_typ_len,
-											   nulls_first,
-											   path->scankey_attno);
 	return &skip_plan->scan.plan;
 }
 
@@ -204,15 +307,36 @@ find_aggrefs_walker(Node *node, FindAggrefsContext *context)
 }
 #endif
 
+static Expr *
+get_distint_clause_expr(PlannerInfo *root, SortGroupClause *distinct_clause)
+{
+	Node *expr = get_sortgroupclause_expr(distinct_clause, root->parse->targetList);
+
+	/* we ignore any columns that can be constified to allow for cases like DISTINCT 'abc',
+	 * column */
+	if (IsA(estimate_expression_value(root, expr), Const))
+		return NULL;
+
+	/* We ignore binary-compatible relabeling */
+	Expr *tlexpr = (Expr *) expr;
+	while (tlexpr && IsA(tlexpr, RelabelType))
+		tlexpr = ((RelabelType *) tlexpr)->arg;
+
+	if (!IsA(tlexpr, Var))
+		return NULL;
+
+	return tlexpr;
+}
+
 /* We can get upper path Distinct expression once for upper path,
  * rather than repeat this check for each child path of an upper path input
  */
-static Expr *
+static List *
 get_upper_distinct_expr(PlannerInfo *root, UpperRelationKind stage)
 {
 	ListCell *lc;
-	int num_vars = 0;
 	Expr *tlexpr = NULL;
+	List *result = NULL;
 
 	if (stage == UPPERREL_DISTINCT && root->parse->distinctClause)
 	{
@@ -220,47 +344,51 @@ get_upper_distinct_expr(PlannerInfo *root, UpperRelationKind stage)
 		 * Examples of queries with 1 Distinct key but multiple target entries:
 		 * SELECT dev, dev FROM t; SELECT 1, dev FROM t; SELECT dev, time FROM t WHERE time = 100;
 		 */
-		if (root->distinct_pathkeys)
-		{
-			SortGroupClause *distinct_clause = NULL;
+		SortGroupClause *distinct_clause = NULL;
 #if PG16_GE
-			distinct_clause = linitial(root->processed_distinctClause);
-#else
-			PathKey *pathkey = linitial(root->distinct_pathkeys);
-			if (pathkey->pk_eclass->ec_sortref)
-			{
-				foreach (lc, root->parse->distinctClause)
-				{
-					SortGroupClause *clause = lfirst_node(SortGroupClause, lc);
-					if (clause->tleSortGroupRef == pathkey->pk_eclass->ec_sortref)
-					{
-						distinct_clause = clause;
-						break;
-					}
-				}
-			}
-			/* We can get PathKey with ec_sortref = 0 in PG15
-			 * when False filter is not pushed into a relation with distinct column (i.e. it's on
-			 * top of a join), so need to support this case in PG15
-			 */
+		foreach (lc, root->processed_distinctClause)
+		{
+			distinct_clause = (SortGroupClause *) lfirst(lc);
+			tlexpr = get_distint_clause_expr(root, distinct_clause);
+			if (tlexpr)
+				result = lappend(result, tlexpr);
 			else
 				return NULL;
-#endif
-			Node *expr = get_sortgroupclause_expr(distinct_clause, root->parse->targetList);
-
-			/* we ignore any columns that can be constified to allow for cases like DISTINCT 'abc',
-			 * column */
-			if (IsA(estimate_expression_value(root, expr), Const))
-				return NULL;
-
-			/* We ignore binary-compatible relabeling */
-			tlexpr = (Expr *) expr;
-			while (tlexpr && IsA(tlexpr, RelabelType))
-				tlexpr = ((RelabelType *) tlexpr)->arg;
-
-			num_vars = 1;
 		}
-#if PG16_LT
+#else
+		if (root->distinct_pathkeys)
+		{
+			foreach (lc, root->distinct_pathkeys)
+			{
+				PathKey *pathkey = (PathKey *) lfirst(lc);
+				if (pathkey->pk_eclass->ec_sortref)
+				{
+					foreach (lc, root->parse->distinctClause)
+					{
+						SortGroupClause *clause = lfirst_node(SortGroupClause, lc);
+						if (clause->tleSortGroupRef == pathkey->pk_eclass->ec_sortref)
+						{
+							distinct_clause = clause;
+							break;
+						}
+					}
+					if (!distinct_clause)
+						return NULL;
+				}
+				/* We can get PathKey with ec_sortref = 0 in PG15
+				 * when False filter is not pushed into a relation with distinct column (i.e. it's
+				 * on top of a join), so need to support this case in PG15
+				 */
+				else
+					return NULL;
+
+				tlexpr = get_distint_clause_expr(root, distinct_clause);
+				if (tlexpr)
+					result = lappend(result, tlexpr);
+				else
+					return NULL;
+			}
+		}
 		/* In PG16+ we use LIMIT instead of UpperUniquePath for (numkeys = 0),
 		 * but in PG15- we would still create UpperUniquePath for (numkeys = 0), so handle this case
 		 * here
@@ -269,22 +397,12 @@ get_upper_distinct_expr(PlannerInfo *root, UpperRelationKind stage)
 		{
 			foreach (lc, root->parse->distinctClause)
 			{
-				SortGroupClause *clause = lfirst_node(SortGroupClause, lc);
-				Node *expr = get_sortgroupclause_expr(clause, root->parse->targetList);
-
-				/* we ignore any columns that can be constified to allow for cases like DISTINCT
-				 * 'abc', column */
-				if (IsA(estimate_expression_value(root, expr), Const))
-					continue;
-
-				num_vars++;
-				if (num_vars > 1)
+				distinct_clause = lfirst_node(SortGroupClause, lc);
+				tlexpr = get_distint_clause_expr(root, distinct_clause);
+				if (tlexpr)
+					result = lappend(result, tlexpr);
+				else
 					return NULL;
-
-				/* We ignore binary-compatible relabeling */
-				tlexpr = (Expr *) expr;
-				while (tlexpr && IsA(tlexpr, RelabelType))
-					tlexpr = ((RelabelType *) tlexpr)->arg;
 			}
 		}
 #endif
@@ -325,7 +443,9 @@ get_upper_distinct_expr(PlannerInfo *root, UpperRelationKind stage)
 				 */
 				if (root->processed_groupClause)
 				{
-					/* Should have bailed out on gby exprs > 1 earlier */
+					/* Should have bailed out on gby exprs > 1 earlier
+					 * Only 1-key SkipScan is supported for distinct aggregates
+					 */
 					Assert(list_length(root->processed_groupClause) == 1);
 					SortGroupClause *sortcl =
 						(SortGroupClause *) linitial(root->processed_groupClause);
@@ -333,11 +453,12 @@ get_upper_distinct_expr(PlannerInfo *root, UpperRelationKind stage)
 					if (!equal(gbykey, expr))
 						return NULL;
 				}
+
 				/* Found a valid distinct agg over a valid Var */
 				if (!tlexpr)
 				{
 					tlexpr = expr;
-					num_vars = 1;
+					result = lappend(result, tlexpr);
 				}
 			}
 			else
@@ -347,14 +468,7 @@ get_upper_distinct_expr(PlannerInfo *root, UpperRelationKind stage)
 		}
 	}
 #endif
-	if (num_vars != 1)
-		return NULL;
-
-	/* SkipScan on expressions not supported */
-	if (!tlexpr || !IsA(tlexpr, Var))
-		return NULL;
-
-	return tlexpr;
+	return result;
 }
 
 static void
@@ -379,21 +493,21 @@ obtain_upper_distinct_path(PlannerInfo *root, RelOptInfo *output_rel, DistinctPa
 			{
 				UpperUniquePath *unique = (UpperUniquePath *) lfirst_node(UpperUniquePath, lc);
 
-				/* currently we do not handle DISTINCT on more than one key. To do so,
-				 * we would need to break down the SkipScan into subproblems: first
+				/* We can handle DISTINCT on more than one key if all keys are guaranteed not-nulls.
+				 * To do so, we break down the SkipScan into subproblems: first
 				 * find the minimal tuple then for each prefix find all unique suffix
 				 * tuples. For instance, if we are searching over (int, int), we would
 				 * first find (0, 0) then find (0, N) for all N in the domain, then
 				 * find (1, N), then (2, N), etc
 				 */
-				if (unique->numkeys > 1)
+				if (!ts_guc_enable_multikey_skip_scan && unique->numkeys > 1)
 					return;
 
 #if PG16_GE
 				/* since PG16+ we no longer create UpperUniquePath with 0 numkeys,
 				 * we create LIMIT path instead, so shouldn't be here with 0 numkeys
 				 */
-				Assert(unique->numkeys == 1);
+				Assert(unique->numkeys >= 1);
 #endif
 				dpinfo->unique_path = (Path *) unique;
 				break;
@@ -408,7 +522,7 @@ obtain_upper_distinct_path(PlannerInfo *root, RelOptInfo *output_rel, DistinctPa
 		if (!ts_guc_enable_skip_scan_for_distinct_aggregates)
 			return;
 
-		/* Cannot apply SkipScan to more than one key */
+		/* Cannot apply SkipScan to distinct aggregates with more than one key */
 		if (list_length(root->group_pathkeys) > 1)
 			return;
 
@@ -566,7 +680,7 @@ tsl_skip_scan_paths_add(PlannerInfo *root, RelOptInfo *input_rel, RelOptInfo *ou
 			has_caa = true;
 		}
 
-		if (IsA(subpath, IndexPath) || ts_is_decompress_chunk_path(subpath))
+		if (IsA(subpath, IndexPath) || ts_is_columnar_scan_path(subpath))
 		{
 			subpath = (Path *) skip_scan_path_create(root, subpath, &dpinfo);
 			if (!subpath)
@@ -688,8 +802,97 @@ tsl_skip_scan_paths_add(PlannerInfo *root, RelOptInfo *input_rel, RelOptInfo *ou
 	}
 }
 
+#if PG17_LT
+static bool
+attr_is_notnull(Oid relid, AttrNumber attno)
+{
+	HeapTuple tp = SearchSysCache2(ATTNUM, ObjectIdGetDatum(relid), Int16GetDatum(attno));
+	if (!HeapTupleIsValid(tp))
+		return false;
+	Form_pg_attribute att_tup = (Form_pg_attribute) GETSTRUCT(tp);
+	bool result = att_tup->attnotnull;
+	ReleaseSysCache(tp);
+	return result;
+}
+#endif
+
+/* Check if skip key is guaranteed not-null */
+static void
+check_notnull_skipkey(SkipKeyInfo *skinfo, Path *child_path, IndexPath *index_path)
+{
+	ListCell *l;
+	/* Quickly look through index clauses on this skip key */
+	foreach (l, index_path->indexclauses)
+	{
+		IndexClause *ic = (IndexClause *) lfirst(l);
+		/* index quals are ordered by indexcol, nothing to see if we've passed our indexcol */
+		if (ic->indexcol > skinfo->scankey_attno - 1)
+			break;
+
+		/* We may have row comparison with skip key not being a leading col,
+		 * like (col, skipcol) > (3, 5), but it can allow NULL skipcols to pass if (col>3) is true,
+		 * so for row comparisons we will only look at leading "indexcol" and not at "indexcols".
+		 */
+		if (ic->indexcol == skinfo->scankey_attno - 1)
+		{
+			/* Any simple index qual but "isNull" filters out nulls,
+			 * including "lossy" index quals extracted from index clauses.
+			 */
+			ListCell *lc;
+			foreach (lc, ic->indexquals)
+			{
+				RestrictInfo *iqual = (RestrictInfo *) lfirst(lc);
+				if (!(IsA(iqual->clause, NullTest) &&
+					  ((NullTest *) iqual->clause)->nulltesttype == IS_NULL))
+				{
+					skinfo->notnull = true;
+					return;
+				}
+			}
+		}
+	}
+
+	/* Otherwise look at all non-indexqual index filters on the key (like (key+1)>5) to see if they
+	 * filter out NULLs */
+	RelOptInfo *indexed_rel = index_path->path.parent;
+	foreach (l, index_path->indexinfo->indrestrictinfo)
+	{
+		RestrictInfo *ri = castNode(RestrictInfo, lfirst(l));
+		Bitmapset *clause_attnos = NULL;
+		pull_varattnos((Node *) ri->clause, indexed_rel->relid, &clause_attnos);
+		if (bms_is_member(skinfo->indexed_column_attno - FirstLowInvalidHeapAttributeNumber,
+						  clause_attnos))
+		{
+			if (!contain_nonstrict_functions((Node *) ri->clause))
+			{
+				skinfo->notnull = true;
+				return;
+			}
+		}
+	}
+
+	/* Failing that, look at filters not pushed down into index (like col1+col2>1) to see if they
+	 * filter out NULLs */
+	RelOptInfo *child_rel = child_path->parent;
+	foreach (l, child_rel->baserestrictinfo)
+	{
+		RestrictInfo *ri = castNode(RestrictInfo, lfirst(l));
+		Bitmapset *clause_attnos = NULL;
+		pull_varattnos((Node *) ri->clause, child_rel->relid, &clause_attnos);
+		if (bms_is_member(skinfo->distinct_attno - FirstLowInvalidHeapAttributeNumber,
+						  clause_attnos))
+		{
+			if (!contain_nonstrict_functions((Node *) ri->clause))
+			{
+				skinfo->notnull = true;
+				return;
+			}
+		}
+	}
+}
+
 static IndexPath *
-get_compressed_index_path(DecompressChunkPath *dcpath)
+get_compressed_index_path(ColumnarScanPath *dcpath)
 {
 	Path *compressed_path = linitial(dcpath->custom_path.custom_paths);
 	if (IsA(compressed_path, IndexPath))
@@ -712,12 +915,12 @@ skip_scan_path_create(PlannerInfo *root, Path *child_path, DistinctPathInfo *dpi
 	{
 		index_path = castNode(IndexPath, child_path);
 	}
-	else if (ts_is_decompress_chunk_path(child_path))
+	else if (ts_is_columnar_scan_path(child_path))
 	{
 		if (!ts_guc_enable_compressed_skip_scan)
 			return NULL;
 
-		DecompressChunkPath *dcpath = (DecompressChunkPath *) child_path;
+		ColumnarScanPath *dcpath = (ColumnarScanPath *) child_path;
 		index_path = get_compressed_index_path(dcpath);
 	}
 	if (!index_path)
@@ -745,16 +948,38 @@ skip_scan_path_create(PlannerInfo *root, Path *child_path, DistinctPathInfo *dpi
 	 * free so reusing the IndexPath here is safe. */
 	skip_scan_path->index_path = index_path;
 
-	Var *dvar = get_distinct_var(root, dpinfo, child_path, skip_scan_path);
+	ListCell *lc;
+	int sk_no = 0;
+	int num_skipkeys = list_length(dpinfo->distinct_expr);
+	foreach (lc, dpinfo->distinct_expr)
+	{
+		Expr *dexpr = (Expr *) lfirst(lc);
 
-	if (!dvar)
-		return NULL;
+		/* Placeholder for skip key attributes */
+		SkipKeyInfo *skinfo = palloc(sizeof(SkipKeyInfo));
+		Var *dvar = get_distinct_var(root, dexpr, index_path, child_path, skinfo);
+		if (!dvar)
+		{
+			pfree(skinfo);
+			return NULL;
+		}
 
-	skip_scan_path->distinct_var = dvar;
+		/* build skip qual this may fail if we cannot look up the operator */
+		if (!build_skip_qual(root, skinfo, index_path, dvar, (++sk_no) < num_skipkeys))
+		{
+			pfree(skinfo);
+			return NULL;
+		}
+		if (!skinfo->notnull)
+			check_notnull_skipkey(skinfo, child_path, index_path);
 
-	/* build skip qual this may fail if we cannot look up the operator */
-	if (!build_skip_qual(root, skip_scan_path, index_path, dvar))
-		return NULL;
+		/* Multikey SkipScan is only supported in not-null mode */
+		if (!skinfo->notnull && num_skipkeys > 1)
+			return NULL;
+
+		skip_scan_path->dvars = lappend(skip_scan_path->dvars, dvar);
+		skip_scan_path->skipkeyinfo = lappend(skip_scan_path->skipkeyinfo, skinfo);
+	}
 
 	/* We have valid SkipScanPath: now we can cost it */
 	double startup = child_path->startup_cost;
@@ -779,8 +1004,8 @@ skip_scan_path_create(PlannerInfo *root, Path *child_path, DistinctPathInfo *dpi
 	 */
 	if (ndistinct > 1)
 	{
-		List *dist_exprs = list_make1(dvar);
-		ndistinct = Max(1, floor(estimate_num_groups(root, dist_exprs, ndistinct, NULL, NULL)));
+		ndistinct =
+			Max(1, floor(estimate_num_groups(root, skip_scan_path->dvars, ndistinct, NULL, NULL)));
 	}
 	skip_scan_path->cpath.path.rows = ndistinct;
 
@@ -822,6 +1047,17 @@ skip_scan_path_create(PlannerInfo *root, Path *child_path, DistinctPathInfo *dpi
 		}
 	}
 
+	/* Heuristic for accounting for previous key shifts in multikey SkipScan
+	 * In general, we will have (k_1*k_2*..*k_N + k_1*..*k_N-1 + ... + k_1) shifts
+	 * where numdistinct = k_1*k_2*..*k_N and for each key "k_i" we have "k_i+1" distinct values.
+	 *
+	 * Worst case scenario is when k_1 = numdistinct and the rest =1, then we'll have (numdistinct *
+	 * N) shifts. If it's uniform for each key i.e. numdistinct = k^N we'll have less than
+	 * (numdistinct * 2) shifts. We pick something in the middle to avoid evaluating k_i for each
+	 * key.
+	 */
+	int numkeys_multiplier = 1.0 + (num_skipkeys - 1) * 0.5;
+
 	/* We calculate SkipScan cost as ndistinct * startup_cost + (ndistinct/rows) * total_cost
 	 * ndistinct * startup_cost is to account for the rescans we have to do and since startup
 	 * cost for indexes does not include page access cost we add a fraction of the total cost
@@ -839,11 +1075,11 @@ skip_scan_path_create(PlannerInfo *root, Path *child_path, DistinctPathInfo *dpi
 		skip_scan_path->cpath.path.startup_cost = startup;
 		if (indexscan_rows > 1)
 			skip_scan_path->cpath.path.total_cost =
-				ndistinct * startup + (ndistinct / rows) * total;
+				ndistinct * startup * numkeys_multiplier + (ndistinct / rows) * total;
 		else
 			skip_scan_path->cpath.path.total_cost = startup;
 	}
-	/* For (SkipScan <- DecompressChunks <- compressed IndexScan) scenario
+	/* For (SkipScan <- ColumnarScan <- compressed IndexScan) scenario
 	 * we will estimate cost as (ndistinct * costs( child_path LIMIT 1 OFFSET x))
 	 * i.e. as if we computed "ndistinct" LIMIT 1 queries on the "child_path" after initial setup.
 	 * If there is no qual above IndexScan, then OFFSET=0 (we don't need to scan tuples to pass qual
@@ -865,7 +1101,8 @@ skip_scan_path_create(PlannerInfo *root, Path *child_path, DistinctPathInfo *dpi
 
 		skip_scan_path->cpath.path.startup_cost = startup;
 		if (indexscan_rows > 1)
-			skip_scan_path->cpath.path.total_cost = startup + (total - startup) * ndistinct;
+			skip_scan_path->cpath.path.total_cost =
+				startup + (total - startup) * ndistinct * numkeys_multiplier;
 		else
 			skip_scan_path->cpath.path.total_cost = startup;
 	}
@@ -881,30 +1118,38 @@ skip_scan_path_create(PlannerInfo *root, Path *child_path, DistinctPathInfo *dpi
 
 /* Extract the Var to use for the SkipScan and do attno mapping if required. */
 static Var *
-get_distinct_var(PlannerInfo *root, DistinctPathInfo *dpinfo, Path *child_path,
-				 SkipScanPath *skip_scan_path)
+get_distinct_var(PlannerInfo *root, Expr *tlexpr, IndexPath *index_path, Path *child_path,
+				 SkipKeyInfo *skinfo)
 {
 	RelOptInfo *rel = child_path->parent;
-	IndexPath *index_path = skip_scan_path->index_path;
 	RelOptInfo *indexed_rel = index_path->path.parent;
-	Expr *tlexpr = dpinfo->distinct_expr;
-
-	if (!tlexpr || !IsA(tlexpr, Var))
-		return NULL;
 
+	Assert(tlexpr && IsA(tlexpr, Var));
 	Var *var = castNode(Var, tlexpr);
 
+	RangeTblEntry *ht_rte = planner_rt_fetch(var->varno, root);
+
+	/* check whether a skip var is declared NOT NULL
+	 *  it's enough to check hypertable for NOT NULL
+	 *  as NOT NULL constraint will be propagated to and checked on all chunks
+	 */
+#if PG17_LT
+	skinfo->notnull = attr_is_notnull(ht_rte->relid, var->varattno);
+#else
+	RelOptInfo *baserel = ((Index) var->varno == rel->relid ? rel : rel->parent);
+	skinfo->notnull = bms_is_member(var->varattno, baserel->notnullattnums);
+#endif
+
 	/* If we are dealing with a hypertable Var extracted from distinctClause will point to
 	 * the parent hypertable while the IndexPath will be on a Chunk.
 	 * For a normal PG table they point to the same relation and we are done here. */
 	if ((Index) var->varno == rel->relid)
 	{
 		/* Get attribute number for distinct column on a normal PG table */
-		skip_scan_path->indexed_column_attno = var->varattno;
+		skinfo->indexed_column_attno = var->varattno;
 		return var;
 	}
 
-	RangeTblEntry *ht_rte = planner_rt_fetch(var->varno, root);
 	RangeTblEntry *chunk_rte = planner_rt_fetch(rel->relid, root);
 	RangeTblEntry *indexed_rte =
 		(indexed_rel == rel ? chunk_rte : planner_rt_fetch(indexed_rel->relid, root));
@@ -918,20 +1163,20 @@ get_distinct_var(PlannerInfo *root, DistinctPathInfo *dpinfo, Path *child_path,
 	var->varattno = get_attnum(chunk_rte->relid, attname);
 
 	/* Get attribute number for distinct column on a compressed chunk */
-	if (ts_is_decompress_chunk_path(child_path))
+	if (ts_is_columnar_scan_path(child_path))
 	{
 		/* distinct column has to be a segmentby column */
-		DecompressChunkPath *dcpath = (DecompressChunkPath *) child_path;
+		ColumnarScanPath *dcpath = (ColumnarScanPath *) child_path;
 		if (!bms_is_member(var->varattno, dcpath->info->chunk_segmentby_attnos))
 		{
 			return NULL;
 		}
-		skip_scan_path->indexed_column_attno = get_attnum(indexed_rte->relid, attname);
+		skinfo->indexed_column_attno = get_attnum(indexed_rte->relid, attname);
 	}
 	/* Get attribute number for distinct column on an uncompressed chunk */
 	else
 	{
-		skip_scan_path->indexed_column_attno = var->varattno;
+		skinfo->indexed_column_attno = var->varattno;
 	}
 
 	var->varno = rel->relid;
@@ -954,7 +1199,7 @@ build_subpath(PlannerInfo *root, List *subpaths, DistinctPathInfo *dpinfo, List
 	foreach (lc, subpaths)
 	{
 		Path *child = lfirst(lc);
-		if (IsA(child, IndexPath) || ts_is_decompress_chunk_path(child))
+		if (IsA(child, IndexPath) || ts_is_columnar_scan_path(child))
 		{
 			if (top_pathkeys && !pathkeys_contained_in(top_pathkeys, child->pathkeys))
 				continue;
@@ -981,7 +1226,8 @@ build_subpath(PlannerInfo *root, List *subpaths, DistinctPathInfo *dpinfo, List
 }
 
 static bool
-build_skip_qual(PlannerInfo *root, SkipScanPath *skip_scan_path, IndexPath *index_path, Var *var)
+build_skip_qual(PlannerInfo *root, SkipKeyInfo *skinfo, IndexPath *index_path, Var *var,
+				bool build_eqop)
 {
 	IndexOptInfo *info = index_path->indexinfo;
 	Oid column_type = exprType((Node *) var);
@@ -1004,16 +1250,16 @@ build_skip_qual(PlannerInfo *root, SkipScanPath *skip_scan_path, IndexPath *inde
 	 * Since a is always 2 due to the WHERE clause we can create the correct ordering for the
 	 * ORDER BY with an index that does not include the a column and only includes the time column.
 	 */
-	int idx_key = get_idx_key(index_path->indexinfo, skip_scan_path->indexed_column_attno);
+	int idx_key = get_idx_key(index_path->indexinfo, skinfo->indexed_column_attno);
 	if (idx_key < 0)
 		return false;
 
 	/* sk_attno of the skip qual */
-	skip_scan_path->scankey_attno = idx_key + 1;
+	skinfo->scankey_attno = idx_key + 1;
 
-	skip_scan_path->distinct_attno = var->varattno;
-	skip_scan_path->distinct_by_val = tce->typbyval;
-	skip_scan_path->distinct_typ_len = tce->typlen;
+	skinfo->distinct_attno = var->varattno;
+	skinfo->distinct_by_val = tce->typbyval;
+	skinfo->distinct_typ_len = tce->typlen;
 
 	int16 strategy = info->reverse_sort[idx_key] ? BTLessStrategyNumber : BTGreaterStrategyNumber;
 	if (index_path->indexscandir == BackwardScanDirection)
@@ -1026,6 +1272,13 @@ build_skip_qual(PlannerInfo *root, SkipScanPath *skip_scan_path, IndexPath *inde
 	Oid comparator =
 		get_opfamily_member(info->sortopfamily[idx_key], column_type, column_type, strategy);
 
+	Oid eqop = InvalidOid;
+	if (build_eqop)
+		eqop = get_opfamily_member(info->sortopfamily[idx_key],
+								   column_type,
+								   column_type,
+								   BTEqualStrategyNumber);
+
 	/* If there is no exact operator match for the column type we have here check
 	 * if we can coerce to the type of the operator class. */
 	if (!OidIsValid(comparator))
@@ -1036,6 +1289,12 @@ build_skip_qual(PlannerInfo *root, SkipScanPath *skip_scan_path, IndexPath *inde
 				get_opfamily_member(info->sortopfamily[idx_key], opcintype, opcintype, strategy);
 			if (!OidIsValid(comparator))
 				return false;
+
+			if (build_eqop)
+				eqop = get_opfamily_member(info->sortopfamily[idx_key],
+										   opcintype,
+										   opcintype,
+										   BTEqualStrategyNumber);
 			need_coerce = true;
 		}
 		else
@@ -1044,7 +1303,7 @@ build_skip_qual(PlannerInfo *root, SkipScanPath *skip_scan_path, IndexPath *inde
 
 	Const *prev_val = makeNullConst(need_coerce ? opcintype : column_type, -1, column_collation);
 	Expr *current_val = (Expr *) makeVar(info->rel->relid /*varno*/,
-										 skip_scan_path->indexed_column_attno /*varattno*/,
+										 skinfo->indexed_column_attno /*varattno*/,
 										 column_type /*vartype*/,
 										 -1 /*vartypmod*/,
 										 column_collation /*varcollid*/,
@@ -1070,8 +1329,9 @@ build_skip_qual(PlannerInfo *root, SkipScanPath *skip_scan_path, IndexPath *inde
 										  InvalidOid /*opcollid*/,
 										  info->indexcollations[idx_key] /*inputcollid*/);
 	set_opfuncid(castNode(OpExpr, comparison_expr));
+	skinfo->eqcomp = (build_eqop ? get_opcode(eqop) : InvalidOid);
 
-	skip_scan_path->skip_clause = make_simple_restrictinfo(root, comparison_expr);
+	skinfo->skip_clause = make_simple_restrictinfo(root, comparison_expr);
 
 	return true;
 }
